@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: avr.c,v 1.75 2008/07/26 22:53:39 joerg_wunsch Exp $ */
+/* $Id: avr.c 821 2009-06-24 21:32:12Z joerg_wunsch $ */
 
 #include "ac_cfg.h"
 
@@ -171,8 +171,13 @@ int avr_read(PROGRAMMER * pgm, AVRPART * p, char * memtype, int size,
    */
   memset(buf, 0xff, size);
 
-  if ((strcmp(mem->desc, "flash")==0) || (strcmp(mem->desc, "eeprom")==0) ||
-      (strcmp(mem->desc, "boot")==0)) {
+  if ((strcmp(mem->desc, "eeprom")==0) || 
+      (strcmp(mem->desc, "flash")==0) ||
+      (strcmp(mem->desc, "application")==0) ||
+      (strcmp(mem->desc, "apptable")==0) ||
+      (strcmp(mem->desc, "boot")==0) ||
+      (strcmp(mem->desc, "usersig")==0) ||
+      (strcmp(mem->desc, "prodsig")==0)) {
     if (pgm->paged_load != NULL && mem->page_size != 0) {
       /*
        * the programmer supports a paged mode read, perhaps more
@@ -569,8 +574,12 @@ int avr_write(PROGRAMMER * pgm, AVRPART * p, char * memtype, int size,
             progbuf, wsize);
   }
 
-  if ((strcmp(m->desc, "flash")==0) || (strcmp(m->desc, "eeprom")==0) ||
-      (strcmp(m->desc, "boot")==0)) {
+  if ((strcmp(m->desc, "application")==0) || 
+      (strcmp(m->desc, "apptable")==0) ||
+      (strcmp(m->desc, "boot")==0) ||
+      (strcmp(m->desc, "flash")==0) ||
+      (strcmp(m->desc, "prodsig")==0) ||
+      (strcmp(m->desc, "usersig")==0)) {
     if (pgm->paged_write != NULL && m->page_size != 0) {
       /*
        * the programmer supports a paged mode write, perhaps more
