@@ -41,66 +41,110 @@
 #if defined(HAVE_LIBUSB)      // we use LIBUSB to talk to the board
 #include <usb.h>
 
+#ifndef HAVE_UINT_T
 typedef	unsigned int	uint_t;
+#endif
+#ifndef HAVE_ULONG_T
 typedef	unsigned long	ulong_t;
+#endif
 
 extern int avr_write_byte_default ( PROGRAMMER* pgm, AVRPART* p,
 				    AVRMEM* mem, ulong_t addr,
 				    unsigned char data );
-static	usb_dev_handle*	usb_handle;
-static	int		sck_period;
-static	int		chunk_size;
+/*
+ * Private data for this programmer.
+ */
+struct pdata
+{
+  usb_dev_handle *usb_handle;
+  int sck_period;
+  int chunk_size;
+  int retries;
+};
 
+#define PDATA(pgm) ((struct pdata *)(pgm->cookie))
 
 // ----------------------------------------------------------------------
 
+static void usbtiny_setup(PROGRAMMER * pgm)
+{
+  if ((pgm->cookie = malloc(sizeof(struct pdata))) == 0) {
+    fprintf(stderr,
+	    "%s: usbtiny_setup(): Out of memory allocating private data\n",
+	    progname);
+    exit(1);
+  }
+  memset(pgm->cookie, 0, sizeof(struct pdata));
+}
+
+static void usbtiny_teardown(PROGRAMMER * pgm)
+{
+  free(pgm->cookie);
+}
+
 // Wrapper for simple usb_control_msg messages
-static int usb_control (unsigned int requestid, unsigned int val, unsigned int index )
+static int usb_control (PROGRAMMER * pgm,
+			unsigned int requestid, unsigned int val, unsigned int index )
 {
   int nbytes;
-  nbytes = usb_control_msg( usb_handle,
+  nbytes = usb_control_msg( PDATA(pgm)->usb_handle,
 			    USB_ENDPOINT_IN | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
 			    requestid,
 			    val, index,           // 2 bytes each of data
 			    NULL, 0,              // no data buffer in control messge
 			    USB_TIMEOUT );        // default timeout
   if(nbytes < 0){
-    fprintf(stderr, "%s: error: usbtiny_transmit: %s\n", progname, usb_strerror());
-    exit(1);
+    fprintf(stderr, "\n%s: error: usbtiny_transmit: %s\n", progname, usb_strerror());
+    return -1;
   }
 
   return nbytes;
 }
 
 // Wrapper for simple usb_control_msg messages to receive data from programmer
-static int usb_in (unsigned int requestid, unsigned int val, unsigned int index,
+static int usb_in (PROGRAMMER * pgm,
+		   unsigned int requestid, unsigned int val, unsigned int index,
 		   unsigned char* buffer, int buflen, int bitclk )
 {
   int nbytes;
   int timeout;
+  int i;
 
   // calculate the amout of time we expect the process to take by
   // figuring the bit-clock time and buffer size and adding to the standard USB timeout.
   timeout = USB_TIMEOUT + (buflen * bitclk) / 1000;
 
-  nbytes = usb_control_msg( usb_handle,
-			    USB_ENDPOINT_IN | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
-			    requestid,
-			    val, index,
-			    (char *)buffer, buflen,
-			    timeout);
-  if (nbytes != buflen) {
-    fprintf(stderr, "%s: error: usbtiny_receive: %s (expected %d, got %d)\n",
-	    progname, usb_strerror(), buflen, nbytes);
-    exit(1);
+  for (i = 0; i < 10; i++) {
+    nbytes = usb_control_msg( PDATA(pgm)->usb_handle,
+			      USB_ENDPOINT_IN | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
+			      requestid,
+			      val, index,
+			      (char *)buffer, buflen,
+			      timeout);
+    if (nbytes == buflen) {
+      return nbytes;
+    }
+    PDATA(pgm)->retries++;
   }
+  fprintf(stderr, "\n%s: error: usbtiny_receive: %s (expected %d, got %d)\n",
+          progname, usb_strerror(), buflen, nbytes);
+  return -1;
+}
 
-  return nbytes;
+// Report the number of retries, and reset the counter.
+static void check_retries (PROGRAMMER * pgm, const char* operation)
+{
+  if (PDATA(pgm)->retries > 0 && quell_progress < 2) {
+    printf("%s: %d retries during %s\n", progname,
+           PDATA(pgm)->retries, operation);
+  }
+  PDATA(pgm)->retries = 0;
 }
 
 // Wrapper for simple usb_control_msg messages to send data to programmer
-static int usb_out (unsigned int requestid, unsigned int val, unsigned int index,
-		     unsigned char* buffer, int buflen, int bitclk )
+static int usb_out (PROGRAMMER * pgm,
+		    unsigned int requestid, unsigned int val, unsigned int index,
+		    unsigned char* buffer, int buflen, int bitclk )
 {
   int nbytes;
   int timeout;
@@ -109,16 +153,16 @@ static int usb_out (unsigned int requestid, unsigned int val, unsigned int index
   // figuring the bit-clock time and buffer size and adding to the standard USB timeout.
   timeout = USB_TIMEOUT + (buflen * bitclk) / 1000;
 
-  nbytes = usb_control_msg( usb_handle,
+  nbytes = usb_control_msg( PDATA(pgm)->usb_handle,
 			    USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
 			    requestid,
 			    val, index,
 			    (char *)buffer, buflen,
 			    timeout);
   if (nbytes != buflen) {
-    fprintf(stderr, "%s: error: usbtiny_send: %s (expected %d, got %d)\n",
+    fprintf(stderr, "\n%s: error: usbtiny_send: %s (expected %d, got %d)\n",
 	    progname, usb_strerror(), buflen, nbytes);
-    exit(1);
+    return -1;
   }
 
   return nbytes;
@@ -157,7 +201,7 @@ static	int	usbtiny_open(PROGRAMMER* pgm, char* name)
   usb_find_busses();             // have libusb scan all the usb busses available
   usb_find_devices();            // have libusb scan all the usb devices available
 
-  usb_handle = NULL;
+  PDATA(pgm)->usb_handle = NULL;
 
   // now we iterate through all the busses and devices
   for ( bus = usb_busses; bus; bus = bus->next ) {
@@ -165,10 +209,10 @@ static	int	usbtiny_open(PROGRAMMER* pgm, char* name)
       if (dev->descriptor.idVendor == USBTINY_VENDOR
 	  && dev->descriptor.idProduct == USBTINY_PRODUCT ) {   // found match?
 
-	usb_handle = usb_open(dev);           // attempt to connect to device
+	PDATA(pgm)->usb_handle = usb_open(dev);           // attempt to connect to device
 
 	// wrong permissions or something?
-	if (!usb_handle) {
+	if (!PDATA(pgm)->usb_handle) {
 	  fprintf(stderr, "%s: Warning: cannot open USB device: %s\n",
 		  progname, usb_strerror());
 	  continue;
@@ -177,10 +221,10 @@ static	int	usbtiny_open(PROGRAMMER* pgm, char* name)
     }
   }
 
-  if (!usb_handle) {
+  if (!PDATA(pgm)->usb_handle) {
     fprintf( stderr, "%s: Error: Could not find USBtiny device (0x%x/0x%x)\n",
 	     progname, USBTINY_VENDOR, USBTINY_PRODUCT );
-    exit(1);
+    return -1;
   }
 
   return 0;                  // If we got here, we must have found a good USB device
@@ -189,22 +233,22 @@ static	int	usbtiny_open(PROGRAMMER* pgm, char* name)
 /* Clean up the handle for the usbtiny */
 static	void usbtiny_close ( PROGRAMMER* pgm )
 {
-  if (! usb_handle) {
+  if (! PDATA(pgm)->usb_handle) {
     return;                // not a valid handle, bail!
   }
-  usb_close(usb_handle);   // ask libusb to clean up
-  usb_handle = NULL;
+  usb_close(PDATA(pgm)->usb_handle);   // ask libusb to clean up
+  PDATA(pgm)->usb_handle = NULL;
 }
 
 /* A simple calculator function determines the maximum size of data we can
    shove through a USB connection without getting errors */
-static void usbtiny_set_chunk_size (int period)
+static void usbtiny_set_chunk_size (PROGRAMMER * pgm, int period)
 {
-  chunk_size = CHUNK_SIZE;       // start with the maximum (default)
-  while	(chunk_size > 8 && period > 16) {
+  PDATA(pgm)->chunk_size = CHUNK_SIZE;       // start with the maximum (default)
+  while	(PDATA(pgm)->chunk_size > 8 && period > 16) {
     // Reduce the chunk size for a slow SCK to reduce
     // the maximum time of a single USB transfer.
-    chunk_size >>= 1;
+    PDATA(pgm)->chunk_size >>= 1;
     period >>= 1;
   }
 }
@@ -213,23 +257,28 @@ static void usbtiny_set_chunk_size (int period)
    USBtiny to update itself to the new frequency */
 static int usbtiny_set_sck_period (PROGRAMMER *pgm, double v)
 {
-  sck_period = (int)(v * 1e6 + 0.5);   // convert from us to 'int', the 0.5 is for rounding up
+  PDATA(pgm)->sck_period = (int)(v * 1e6 + 0.5);   // convert from us to 'int', the 0.5 is for rounding up
 
   // Make sure its not 0, as that will confuse the usbtiny
-  if (sck_period < SCK_MIN)
-    sck_period = SCK_MIN;
+  if (PDATA(pgm)->sck_period < SCK_MIN)
+    PDATA(pgm)->sck_period = SCK_MIN;
 
   // We can't go slower, due to the byte-size of the clock variable
-  if  (sck_period > SCK_MAX)
-    sck_period = SCK_MAX;
+  if  (PDATA(pgm)->sck_period > SCK_MAX)
+    PDATA(pgm)->sck_period = SCK_MAX;
 
-  printf( "%s: Setting SCK period to %d usec\n", progname, sck_period );
+  if (verbose) {
+    printf( "%s: Setting SCK period to %d usec\n", progname,
+	    PDATA(pgm)->sck_period );
+  }
 
   // send the command to the usbtiny device.
-  usb_control(USBTINY_POWERUP, sck_period, RESET_LOW);  // MEME: for at90's fix resetstate?
+  // MEME: for at90's fix resetstate?
+  if (usb_control(pgm, USBTINY_POWERUP, PDATA(pgm)->sck_period, RESET_LOW) < 0)
+    return -1;
 
   // with the new speed, we'll have to update how much data we send per usb transfer
-  usbtiny_set_chunk_size(sck_period);
+  usbtiny_set_chunk_size(pgm, PDATA(pgm)->sck_period);
   return 0;
 }
 
@@ -244,13 +293,15 @@ static int usbtiny_initialize (PROGRAMMER *pgm, AVRPART *p )
     usbtiny_set_sck_period(pgm, pgm->bitclock);
   } else {
     // -B option not specified: use default
-    sck_period = SCK_DEFAULT;
+    PDATA(pgm)->sck_period = SCK_DEFAULT;
     if	(verbose) {
       printf( "%s: Using SCK period of %d usec\n",
-	      progname, sck_period );
+	      progname, PDATA(pgm)->sck_period );
     }
-    usb_control( USBTINY_POWERUP, sck_period, RESET_LOW );
-    usbtiny_set_chunk_size(sck_period);
+    if (usb_control(pgm,  USBTINY_POWERUP,
+		    PDATA(pgm)->sck_period, RESET_LOW ) < 0)
+      return -1;
+    usbtiny_set_chunk_size(pgm, PDATA(pgm)->sck_period);
   }
 
   // Let the device wake up.
@@ -259,8 +310,11 @@ static int usbtiny_initialize (PROGRAMMER *pgm, AVRPART *p )
   // Attempt to use the underlying avrdude methods to connect (MEME: is this kosher?)
   if (! usbtiny_avr_op(pgm, p, AVR_OP_PGM_ENABLE, res)) {
     // no response, RESET and try again
-    usb_control(USBTINY_POWERUP, sck_period, RESET_HIGH);
-    usb_control(USBTINY_POWERUP, sck_period, RESET_LOW);
+    if (usb_control(pgm, USBTINY_POWERUP,
+		    PDATA(pgm)->sck_period, RESET_HIGH) < 0 ||
+	usb_control(pgm, USBTINY_POWERUP,
+		    PDATA(pgm)->sck_period, RESET_LOW) < 0)
+      return -1;
     usleep(50000);
     if	( ! usbtiny_avr_op( pgm, p, AVR_OP_PGM_ENABLE, res)) {
       // give up
@@ -273,10 +327,10 @@ static int usbtiny_initialize (PROGRAMMER *pgm, AVRPART *p )
 /* Tell the USBtiny to release the output pins, etc */
 static void usbtiny_powerdown(PROGRAMMER * pgm)
 {
-  if (!usb_handle) {
+  if (!PDATA(pgm)->usb_handle) {
     return;                 // wasn't connected in the first place
   }
-  usb_control(USBTINY_POWERDOWN, 0, 0);      // Send USB control command to device
+  usb_control(pgm, USBTINY_POWERDOWN, 0, 0);      // Send USB control command to device
 }
 
 /* Send a 4-byte SPI command to the USBtinyISP for execution
@@ -286,19 +340,22 @@ static int usbtiny_cmd(PROGRAMMER * pgm, unsigned char cmd[4], unsigned char res
   int nbytes;
 
   // Make sure its empty so we don't read previous calls if it fails
-  memset(res, '\0', sizeof(res) );
+  memset(res, '\0', 4 );
 
-  nbytes = usb_in( USBTINY_SPI,
+  nbytes = usb_in( pgm, USBTINY_SPI,
 		   (cmd[1] << 8) | cmd[0],  // convert to 16-bit words
 		   (cmd[3] << 8) | cmd[2],  //  "
-			res, sizeof(res), 8 * sck_period );
+			res, 4, 8 * PDATA(pgm)->sck_period );
+  if (nbytes < 0)
+    return -1;
+  check_retries(pgm, "SPI command");
   if (verbose > 1) {
     // print out the data we sent and received
     printf( "CMD: [%02x %02x %02x %02x] [%02x %02x %02x %02x]\n",
 	    cmd[0], cmd[1], cmd[2], cmd[3],
 	    res[0], res[1], res[2], res[3] );
   }
-  return ((nbytes == sizeof(res)) &&      // should have read 4 bytes
+  return ((nbytes == 4) &&      // should have read 4 bytes
 	  res[2] == cmd[1]);              // AVR's do a delayed-echo thing
 }
 
@@ -352,7 +409,7 @@ static int usbtiny_paged_load (PROGRAMMER * pgm, AVRPART * p, AVRMEM* m,
   }
 
   for (i = 0; i < n_bytes; i += chunk) {
-    chunk = chunk_size;         // start with the maximum chunk size possible
+    chunk = PDATA(pgm)->chunk_size;         // start with the maximum chunk size possible
 
     // If we want to xmit less than a chunk, thats OK
     if	(chunk > n_bytes-i)
@@ -360,18 +417,23 @@ static int usbtiny_paged_load (PROGRAMMER * pgm, AVRPART * p, AVRMEM* m,
 
     // Send the chunk of data to the USBtiny with the function we want
     // to perform
-    usb_in(function,          // EEPROM or flash
-	   0,                 // delay between SPI commands
-	   i,                 // index
-	   m->buf + i,        // pointer to where we store data
-	   chunk,             // number of bytes
-	   32 * sck_period);  // each byte gets turned into a 4-byte SPI cmd
+    if (usb_in(pgm,
+	       function,          // EEPROM or flash
+	       0,                 // delay between SPI commands
+	       i,                 // index
+	       m->buf + i,        // pointer to where we store data
+	       chunk,             // number of bytes
+	       32 * PDATA(pgm)->sck_period)  // each byte gets turned into a 4-byte SPI cmd
+	< 0) {
                               // usb_in() multiplies this per byte.
+      return -1;
+    }
 
     // Tell avrdude how we're doing to provide user feedback
     report_progress(i + chunk, n_bytes, NULL );
   }
 
+  check_retries(pgm, "read");
   return n_bytes;
 }
 
@@ -400,13 +462,14 @@ static int usbtiny_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   if (! m->paged) {
     // Does this chip not support paged writes?
     i = (m->readback[1] << 8) | m->readback[0];
-    usb_control( USBTINY_POLL_BYTES, i, 0 );
+    if (usb_control(pgm, USBTINY_POLL_BYTES, i, 0 ) < 0)
+      return -1;
     delay = m->max_write_delay;
   }
 
   for (i=0; i < n_bytes; i=next) {
     // start with the max chunk size
-    chunk = chunk_size;
+    chunk = PDATA(pgm)->chunk_size;
 
     // we can only write a page at a time anyways
     if (m->paged && chunk > page_size)
@@ -416,15 +479,18 @@ static int usbtiny_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
     if (chunk > n_bytes-i)
       chunk = n_bytes - i;
 
-    usb_out(function,       // Flash or EEPROM
-	    delay,          // How much to wait between each byte
-	    i,              // Index of data
-	    m->buf + i,     // Pointer to data
-	    chunk,          // Number of bytes to write
-	    32 * sck_period + delay  // each byte gets turned into a
-	                             // 4-byte SPI cmd  usb_in() multiplies
+    if (usb_out(pgm,
+		function,       // Flash or EEPROM
+		delay,          // How much to wait between each byte
+		i,              // Index of data
+		m->buf + i,     // Pointer to data
+		chunk,          // Number of bytes to write
+		32 * PDATA(pgm)->sck_period + delay  // each byte gets turned into a
+	                             // 4-byte SPI cmd  usb_out() multiplies
 	                             // this per byte. Then add the cmd-delay
-	    );
+		) < 0) {
+      return -1;
+    }
 
     next = i + chunk;       // Calculate what address we're at now
     if (m->paged
@@ -460,6 +526,8 @@ extern void usbtiny_initpgm ( PROGRAMMER* pgm )
   pgm->paged_load	= usbtiny_paged_load;
   pgm->paged_write	= usbtiny_paged_write;
   pgm->set_sck_period	= usbtiny_set_sck_period;
+  pgm->setup            = usbtiny_setup;
+  pgm->teardown         = usbtiny_teardown;
 }
 
 #else  /* !HAVE_LIBUSB */
@@ -471,7 +539,7 @@ static int usbtiny_nousb_open(struct programmer_t *pgm, char * name)
   fprintf(stderr, "%s: error: no usb support. Please compile again with libusb installed.\n",
 	  progname);
 
-  exit(1);
+  return -1;
 }
 
 void usbtiny_initpgm(PROGRAMMER * pgm)
